@@ -1,7 +1,7 @@
 /* ========================================================================= *
  *                                                                           *
  *                               OpenMesh                                    *
- *           Copyright (c) 2001-2022, RWTH-Aachen University                 *
+ *           Copyright (c) 2001-2023, RWTH-Aachen University                 *
  *           Department of Computer Graphics and Multimedia                  *
  *                          All rights reserved.                             *
  *                            www.openmesh.org                               *
@@ -92,7 +92,7 @@ _PLYWriter_::_PLYWriter_()
 
 bool
 _PLYWriter_::
-write(const std::string& _filename, BaseExporter& _be, Options _opt, std::streamsize _precision) const
+write(const std::string& _filename, BaseExporter& _be, const Options& _opt, std::streamsize _precision) const
 {
 
   // open file
@@ -106,23 +106,24 @@ write(const std::string& _filename, BaseExporter& _be, Options _opt, std::stream
 
 bool
 _PLYWriter_::
-write(std::ostream& _os, BaseExporter& _be, Options _opt, std::streamsize _precision) const
+write(std::ostream& _os, BaseExporter& _be, const Options& _writeOptions, std::streamsize _precision) const
 {
+
+  options_ = _writeOptions;
+
   // check exporter features
-  if ( !check( _be, _opt ) )
+  if ( !check( _be, options_ ) )
     return false;
 
-
   // check writer features
-  if ( _opt.check(Options::FaceNormal) ) {
+  if ( options_.check(Options::FaceNormal) ) {
     // Face normals are not supported
     // Uncheck these options and output message that
     // they are not written out even though they were requested
-    _opt.unset(Options::FaceNormal);
+    options_.unset(Options::FaceNormal);
     omerr() << "[PLYWriter] : Warning: Face normals are not supported and thus not exported! " << std::endl;
   }
 
-  options_ = _opt;
 
 
   if (!_os.good())
@@ -132,13 +133,13 @@ write(std::ostream& _os, BaseExporter& _be, Options _opt, std::streamsize _preci
     return false;
   }
 
-  if (!_opt.check(Options::Binary))
+  if (!options_.check(Options::Binary))
     _os.precision(_precision);
 
   // write to file
-  bool result = (_opt.check(Options::Binary) ?
-     write_binary(_os, _be, _opt) :
-     write_ascii(_os, _be, _opt));
+  bool result = (options_.check(Options::Binary) ?
+     write_binary(_os, _be, options_) :
+     write_ascii(_os, _be, options_));
 
   return result;
 }
@@ -707,7 +708,7 @@ write_binary(std::ostream& _out, BaseExporter& _be, Options _opt) const
 
 size_t
 _PLYWriter_::
-binary_size(BaseExporter& _be, Options _opt) const
+binary_size(BaseExporter& _be, const Options& _writeOptions) const
 {
   size_t header(0);
   size_t data(0);
@@ -715,7 +716,7 @@ binary_size(BaseExporter& _be, Options _opt) const
   size_t _3ui(3*sizeof(unsigned int));
   size_t _4ui(4*sizeof(unsigned int));
 
-  if ( !_opt.is_binary() )
+  if ( !_writeOptions.is_binary() )
     return 0;
   else
   {
@@ -727,19 +728,19 @@ binary_size(BaseExporter& _be, Options _opt) const
     data   += _be.n_vertices() * _3floats;    // vertex data
   }
 
-  if ( _opt.vertex_has_normal() && _be.has_vertex_normals() )
+  if ( _writeOptions.vertex_has_normal() && _be.has_vertex_normals() )
   {
     header += 1; // N
     data   += _be.n_vertices() * _3floats;
   }
 
-  if ( _opt.vertex_has_color() && _be.has_vertex_colors() )
+  if ( _writeOptions.vertex_has_color() && _be.has_vertex_colors() )
   {
     header += 1; // C
     data   += _be.n_vertices() * _3floats;
   }
 
-  if ( _opt.vertex_has_texcoord() && _be.has_vertex_texcoords() )
+  if ( _writeOptions.vertex_has_texcoord() && _be.has_vertex_texcoords() )
   {
     size_t _2floats(2*sizeof(float));
     header += 2; // ST
@@ -764,8 +765,8 @@ binary_size(BaseExporter& _be, Options _opt) const
   }
 
   // face colors
-  if ( _opt.face_has_color() && _be.has_face_colors() ){
-    if ( _opt.color_has_alpha() )
+  if ( _writeOptions.face_has_color() && _be.has_face_colors() ){
+    if ( _writeOptions.color_has_alpha() )
       data += _be.n_faces() * _4ui;
     else
       data += _be.n_faces() * _3ui;
